//DDR SDRAM common interface 
//Automat to give common interface to DDR SDRAM

// Company: Quenne Electric
// Engineer: Andras Bognar

`timescale 1ps / 1ps

module ddraut(
//interface signals to the DDR SDRAM
	A,		//13bit address
	DQ,		//16bit data
	BA,		//bank adress
	RAS,	//row address
	CAS,	//column address
	WE,		//write enable
	UQM,	//upper data mask
	LQM,	//lower data mask
	UDQS,	//upper data strobe
	LDQS,	//lower data strobe
	CS,		//chip select
	CKE,	//clock enable
//clocks
	SYSCLK,	//clock input 200MHz
	DDRCLK,	//clock for ddr ram 100MHz
//Interface signals to the HOST	
	ADDR,	//address bus
	DATAIN,	//data bus
	DATAOUT,//data bus
	W,		//command: write, not read
	READY,	//execution status
	STR		//strobe for command
	);

	
//Burst length=2
//Burst type=Sequential
//CAS latency=2
//DLL: enabled
	
//DDR SDRAM IF
	output [12:0]A;
	inout [15:0]DQ;
	output [1:0]BA;
	output RAS;
	output CAS;
	output WE;
	output UQM;
	output LQM;
	inout UDQS;
	inout LDQS;
	output CS;
	output CKE;
//CLOCK
	input SYSCLK;			//200MHz clock input
	output DDRCLK;			//100MHz clock for DDR RAM
//HOST IF	
	input [23:0]ADDR;		//16M x32  (24 bit)
	input [31:0]DATAIN;
	output[31:0]DATAOUT;
	input W;
	output READY;
	input STR;
	
//INTERNAL SIGNALS
	//state registers
	reg [7:0]state=0;
	reg ddrclk=0;
	reg dq_dir=0;			//direction: 0-HZ, 1-driven
	reg dqs_dir=0;			//direction: 0-HZ, 1-driven
	
	reg [15:0]delay=0;		//delay register
	reg [15:0]delaycnt=0;	//delay counter
//	reg [7:0]nextstate=0;	//next state register after delay
	reg delaying=0;			//delay is in progress
	
	reg [9:0]refi=0;		//refresh delay counter
	reg datalh=0;			//dataout latch: datalh=0->put out low word
	reg readin=0;			//enable read in automat to read data
	//output registers
	reg [12:0]a;
	reg [1:0]ba;
	reg [15:0]dq=0;
	reg ras;
	reg cas;
	reg we;
	reg uqm;
	reg lqm;
	reg udqs;
	reg ldqs;
	reg cs;
	reg cke;
	reg [31:0]data=0;
	reg ready;
	
//DATA STAGE AUTOMAT
	always@(negedge SYSCLK)
		begin
		if(datalh==0)dq[15:0]<=DATAIN[15:0];
		if(datalh==1)dq[15:0]<=DATAIN[31:16];
		if((readin==1)&&(datalh==0))data[15:0]<=DQ[15:0];
		if((readin==1)&&(datalh==1))data[31:16]<=DQ[15:0];
		end
	
//DDRCLK AUTOMAT
	always@(posedge SYSCLK)
		begin
		ddrclk<=(ddrclk)?0:1;
		end
	
//DELAYER AUTOMAT
	always@(negedge SYSCLK)
		begin
		if(delay==delaycnt)
			begin
			delaying<=0;
			delaycnt<=0;
			end
		else
			begin
			delaying<=1;
			delaycnt<=delaycnt+1;
			end
		end
	
//MAIN AUTOMAT
	always@(posedge SYSCLK)
	  if(!delaying)
		begin

//INITIALIZATION STEPS
		case(state)
		8'h00:	//init state
			begin
			readin<=0;
			a<=13'h0000;
			ba<=2'h0;
			ras<=1;
			cas<=1;
			we<=1;
			uqm<=0;
			lqm<=0;
			udqs<=0;
			ldqs<=0;
			cs<=1;
			cke<=0;
			ready<=0;
			dq_dir<=0;
			dqs_dir<=0;
			
			delay<=16'hA000;	//setup 200us delay
//			nextstate<=8'h02;	//next state after delay
//			state<=8'h01;
			state<=8'h02;
			
			end
//		8'h01:	//waiting state (T=5ns)
//			begin
//			delay<=delay-1;
//			if(delay==16'h0000)state<=nextstate;
//			end
		8'h02:	//point 6. NOP
			begin
			delay<=16'h0000;
			cke<=1;
			if(ddrclk==1)
				begin
				cs<=0;
				state<=8'h03;
				end
			end
		8'h03:	//point 7. PRECHARGE ALL
			begin
			if(ddrclk==1)
				begin
				ras<=0;
				we<=0;
				a[10]<=1;
				state<=8'h04;
				end
			end
		8'h04:	//point 8. tRP (15ns)
			begin
			if(ddrclk==1)
				begin
				ras<=1;
				we<=1;
				delay<=16'h0004;
//				nextstate<=8'h05;
//				state<=8'h01;
				state<=8'h05;
				end
			end
		8'h05:	//point 9. LMR
			begin
			delay<=16'h0000;
			if(ddrclk==1)
				begin
				ba[0]<=1;
				a[10]<=0;
				ras<=0;
				cas<=0;
				we<=0;
				state<=8'h06;
				end
			end
		8'h06:	//10. Wait at least tMRD time; 
			begin
			if(ddrclk==1)
				begin
				ras<=1;
				cas<=1;
				we<=1;
				delay<=16'h0004;
//				nextstate<=8'h07;
				state<=8'h07;
				end
			end
		8'h07:	//11. LMR
			begin
			delay<=16'h0000;
			if(ddrclk==1)
				begin
				ba[0]<=0;
				a[8]<=1;
				a[5]<=1;
				a[0]<=1;
				ras<=0;
				cas<=0;
				we<=0;
				state<=8'h08;
				end
			end
		8'h08:	//12. Wait at least tMRD time; 
			begin
			if(ddrclk==1)
				begin
				ras<=1;
				cas<=1;
				we<=1;
				delay<=16'h0004;
//				nextstate<=8'h09;
				state<=8'h09;
				end			
			end
		8'h09:	//13. Issue a PRECHARGE ALL command.
			begin
			delay<=16'h0000;
			if(ddrclk==1)
				begin
				a[10]<=1;				
				a[8]<=0;
				a[5]<=0;
				a[0]<=0;				
				ras<=0;
				we<=0;
				state<=8'h0A;
				end			
			end
		8'h0A:	//14. tRP time
			begin
			if(ddrclk==1)
				begin
				ras<=1;
				we<=1;
				delay<=16'h0004;
//				nextstate<=8'h0B;
				state<=8'h0B;
				end			
			end
		8'h0B:	//15. Issue an AUTO REFRESH command
			begin
			delay<=16'h0000;
			if(ddrclk==1)
				begin
				ras<=0;
				cas<=0;
				state<=8'h0C;
				end
			end
		8'h0C:	//16. Wait at least tRFC time
			begin
			if(ddrclk==1)
				begin
				ras<=1;
				cas<=1;
				delay<=16'h0014;
//				nextstate<=8'h0D;
				state<=8'h0D;
				end						
			end
		8'h0D:	//17. Issue an AUTO REFRESH command
			begin
			delay<=16'h0000;
			if(ddrclk==1)
				begin
				ras<=0;
				cas<=0;
				state<=8'h0E;
				end			
			end
		8'h0E:	//18. Wait at least tRFC time
			begin
			if(ddrclk==1)
				begin
				ras<=1;
				cas<=1;
				delay<=16'h0014;
//				nextstate<=8'h0F;
				state<=8'h0F;
				end									
			end
		8'h0F:	//19. LMR
			begin
			delay<=16'h0000;
			if(ddrclk==1)
				begin
				a[10]<=0;
				a[5]<=1;
				a[0]<=1;
				ras<=0;
				cas<=0;
				we<=0;
				state<=8'h10;
				end			
			end
		8'h10:	//wait for memory be ready
			begin
			if(ddrclk==1)
				begin
				ras<=1;
				cas<=1;
				we<=1;
				delay<=16'h0200;
//				nextstate<=8'h11;
				state<=8'h11;
				end			
			end
//OPERATION CYCLES
		8'h11:	//waiting for bus command or refresh request
			begin
			delay<=16'h0000;
			refi<=refi+1;
			if(refi==10'h000)state<=8'h12;
			if(STR==1)state<=8'h14;
			end
		8'h12:	//auto refreshing
			begin
			if(ddrclk==1)
				begin
				ras<=0;
				cas<=0;
				state<=8'h13;
				end			
			end
		8'h13:	// Wait at least tRFC time
			begin
			if(ddrclk==1)
				begin
				ras<=1;
				cas<=1;
				delay<=16'h0014;
//				nextstate<=8'h11;
				state<=8'h11;
				end									
			end	
		8'h14:	//command
			begin
			if(W==1)state<=8'h15;
			if(W==0)state<=8'h30;
			end
//WRITE			
		8'h15:	//activate
			begin
			if(ddrclk==1)
				begin
				ras<=0;
				ba[1:0]<=ADDR[23:22];
				a[12:0]<=ADDR[21:10];
				state<=8'h16;
				end
			end
		8'h16:	//tRCD
			begin
			if(ddrclk==1)
				begin
				ras<=1;
				delay<=16'h0004;
//				nextstate<=8'h17;
				state<=8'h17;
				end
			end
		8'h17:	//write
			begin
			delay<=16'h0000;
			if(ddrclk==1)
				begin
				cas<=0;
				we<=0;
				a[12:10]<=3'h1;
				a[9:0]<=ADDR[9:0];
				dq_dir<=1;
				dqs_dir<=1;
				state<=8'h18;
				end
			end
		8'h18:	//put out data high
			begin
			if(ddrclk==1)
				begin
				cas<=1;
				we<=1;
				datalh<=1;
				state<=8'h19;
				end
			end
		8'h19:	//put out data low
			begin
			datalh<=0;
			state<=8'h20;
			ldqs<=1;
			udqs<=1;
			end
		8'h20:
			begin
			ldqs<=0;
			udqs<=0;
			state<=8'h21;
			end
		8'h21:	//tWR+tRP
			begin
			ready<=1;
			dq_dir<=0;
			dqs_dir<=0;
			delay<=16'h0008;
//			nextstate<=8'h22;
			state<=8'h22;
			end
		8'h22:
			begin
			delay<=16'h0000;
			if(STR==0)
				begin
				ready<=0;
				state<=8'h11;
				end
			end
//READ			
		8'h30:	//activate
			begin
			if(ddrclk==1)
				begin
				ras<=0;
				ba[1:0]<=ADDR[23:22];
				a[12:0]<=ADDR[21:10];
				state<=8'h31;
				end
			end
		8'h31:	//tRCD
			begin
			if(ddrclk==1)
				begin
				ras<=1;
				delay<=16'h0004;
//				nextstate<=8'h32;
				state<=8'h32;
				end
			end
		8'h32:	//read
			begin
			delay<=16'h0000;
			if(ddrclk==1)
				begin
				cas<=0;
				a[12:10]<=3'h1;
				a[9:0]<=ADDR[9:0];
				state<=8'h33;
				end			
			end
		8'h33:
			begin
			if(ddrclk==1)
				begin
				cas<=1;
				state<=8'h34;
				end
			end
		8'h34:
			begin
			if(ddrclk==1)state<=8'h35;
			end
		8'h35:	//read in data high
			begin
			readin<=1;
			datalh<=1;
			state<=8'h36;
			end
		8'h36:	//read data low
			begin
			datalh<=0;
			state<=8'h37;
			end
		8'h37:	//tWR+tRP
			begin
			readin<=0;
			ready<=1;
			delay<=16'h0008;
//			nextstate<=8'h38;
			state<=8'h38;
			end
		8'h38:
			begin
			delay<=16'h0000;
			if(STR==0)
				begin
				ready<=0;
				state<=8'h11;
				end
			end
		default:state<=8'h00;
		endcase
		end
		
	 
	assign A=a;
	assign DQ=(dq_dir)?dq:16'bz;
	assign BA=ba;
	assign RAS=ras;
	assign CAS=cas;
	assign WE=we;
	assign UQM=uqm;
	assign LQM=lqm;
	assign UDQS=(dqs_dir)?udqs:1'bz;
	assign LDQS=(dqs_dir)?ldqs:1'bz;
	assign CS=cs;
	assign CKE=cke;
	assign DATAOUT=data;
	assign READY=ready;
	assign DDRCLK=ddrclk;
	
endmodule

/*
INIT
1. Simultaneously apply power to VDD and VDDQ.
2. Apply VREF and then VTT power. VTT must be applied after VDDQ to avoid device latchup,
which may cause permanent damage to the device. Exept for CKE, inputs are not
recognized as valid until after VREF is applied.
3. Assert and hold CKE at a LVCMOS logic LOW. Maintaining an LVCMOS LOW level on
CKE during power-up is required to ensure that the DQ and DQS outputs will be in
the High-Z state, where they will remain until driven in normal operation (by a read
access).
4. Provide stable clock signals.
5. Wait at least 200s.
6. Bring CKE HIGH, and provide at least one NOP or DESELECT command. At this
point, the CKE input changes from a LVCMOS input to a SSTL_2 input only and will
remain a SSTL_2 input unless a power cycle occurs.
7. Perform a PRECHARGE ALL command.
8. Wait at least tRP time; during this time NOPs or DESELECT commands must be given.
9. Using the LMR command, program the extended mode register (E0 = 0 to enable the
DLL and E1 = 0 for normal drive; or E1 = 1 for reduced drive and E2En must be set to
0 [where n = most significant bit]).
10. Wait at least tMRD time; only NOPs or DESELECT commands are allowed.
11. Using the LMR command, program the mode register to set operating parameters
and to reset the DLL. At least 200 clock cycles are required between a DLL reset and
any READ command.
12. Wait at least tMRD time; only NOPs or DESELECT commands are allowed.
13. Issue a PRECHARGE ALL command.
14. Wait at least tRP time; only NOPs or DESELECT commands are allowed.
15. Issue an AUTO REFRESH command. This may be moved prior to step 13.
16. Wait at least tRFC time; only NOPs or DESELECT commands are allowed.
17. Issue an AUTO REFRESH command. This may be moved prior to step 13.
18. Wait at least tRFC time; only NOPs or DESELECT commands are allowed.
19. Although not required by the Micron device, JEDEC requires an LMR command to
clear the DLL bit (set M8 = 0). If an LMR command is issued, the same operating
parameters should be utilized as in step 11.
20. Wait at least tMRD time; only NOPs or DESELECT commands are allowed.
21. At this point the DRAM is ready for any valid command. At least 200 clock cycles with
CKE HIGH are required between step 11 (DLL RESET) and any READ command.
*/